﻿RepTracker = LibStub("AceAddon-3.0"):NewAddon("RepTracker", "AceConsole-3.0", "AceEvent-3.0")

local defaults = {
	faction = {
		showInChat = true,
		dynamic = true,
		sticky = true,
		decrease = false,
		reset = true,
		ZoneToFaction = {},
	},
}

local options = {
	name = "RepTracker",
	handler = RepTracker,
	type = "group",
	args = {
		track = {
			order = 1,
			type = 'input',
			name = 'Current faction tracked:',
			desc = 'Sets which faction is tracked in this zone by default.  <none> to remove the default.',
			get = "GetZone",
			set = "SetZone"
		},
		trackDesc = {
			order = 2,
			type = 'description',
			name = 'Sets which faction is tracked in this zone by default.  <none> to remove the default.'
		},
		optionsHeader = {
			order = 3,
			type = 'header',
			name = 'Options'
		},
		dynamic = {
			order = 4,
			type = 'toggle',
			name = 'Dynamic Tracking',
			desc = 'The tracked reputation changes to reflect the last faction you gained reputation with.',
			get = "IsDynamic",
			set = "ToggleDynamic"
		},
		dynamicDesc = {
			order = 5,
			type = 'description',
			name = 'The tracked reputation changes to reflect the last faction you gained reputation with.'
		},
		sticky = {
			order = 6,
			type = 'toggle',
			name = 'Sticky Rep',
			desc = 'Dynamic tracking changes are permanently stored as the default tracked faction for that zone.',
			get = "IsSticky",
			set = "ToggleSticky",
			disabled = function() return not RepTracker.db.faction.dynamic end
		},
		stickyDesc = {
			order = 7,
			type = 'description',
			name = 'Dynamic tracking changes are permanently stored as the default tracked faction.'
		},
		showInChat = {
			order = 8,
			type = 'toggle',
			name = 'Show In Chat',
			desc = 'Display RepTracker messages in the chat window.',
			get = "IsShowInChat",
			set = "ToggleShowInChat"
		},
		showInChatDesc = {
			order = 9,
			type = 'description',
			name = 'Display RepTracker messages in the chat window.'
		},		
		decrease = {
			order = 10,
			type = 'toggle',
			name = 'Track Decreases',
			desc = 'Track reputation decreases.',
			get = "IsDecrease",
			set = "ToggleDecrease"
		},
		decreaseDesc = {
			order = 11,
			type = 'description',
			name = 'Track reputation decreases.'
		},
		reset = {
			order = 12,
			type = 'toggle',
			name = 'Faction Reset',
			desc = 'Clears the tracked faction upon entering a new zone if that zone does not yet have a faction assigned.',
			get = "IsReset",
			set = "ToggleReset"
		},
		resetDesc = {
			order = 13,
			type = 'description',
			name = 'Clears the tracked faction upon entering a new zone if that zone does not yet have a faction assigned.',
		},
		aboutHeader = {
			order = 14,
			type = 'header',
			name = 'About'
		},
		version = {
			order = 15,
			type = 'description',
			name = 'Version: |cFFFF80802.0.0|r'
		},
		author = {
			order = 16,
			type = 'description',
			name = 'by |cFFFF8080InSo(v)nIaX|r'
		},
	},
}

function RepTracker:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("RepTrackerDB", defaults)
	LibStub("AceConfig-3.0"):RegisterOptionsTable("RepTracker", options)
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("RepTracker", "RepTracker")
	self:RegisterChatCommand("rt", "ChatCommand")
	self:RegisterChatCommand("RepTracker", "ChatCommand")
end

function RepTracker:OnEnable()
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
    CombatTextSetActiveUnit("player")
    if self.db.faction.dynamic then
        self:RegisterEvent("COMBAT_TEXT_UPDATE")
    end
end

function RepTracker:ChatCommand(input)
    if not input or input:trim() == "" then
        InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
    else
        LibStub("AceConfigCmd-3.0").HandleCommand(RepTracker, "rt", "RepTracker", input)
    end
end

function RepTracker:IsShowInChat(info)
	return self.db.faction.showInChat
end

function RepTracker:ToggleShowInChat(info, value)
	self.db.faction.showInChat = value
end

function RepTracker:IsDynamic(info)
	return self.db.faction.dynamic
end

function RepTracker:ToggleDynamic(info, value)
	self.db.faction.dynamic = value
    if (self.db.faction.dynamic) then
        self:RegisterEvent("COMBAT_TEXT_UPDATE")
    else
        self:UnregisterEvent("COMBAT_TEXT_UPDATE")
    end
end

function RepTracker:IsSticky(info)
	return self.db.faction.sticky
end

function RepTracker:ToggleSticky(info, value)
	self.db.faction.sticky = value
end

function RepTracker:IsDecrease(info)
	return self.db.faction.decrease
end

function RepTracker:ToggleDecrease(info, value)
	self.db.faction.decrease = value
end

function RepTracker:IsReset(info)
	return self.db.faction.reset
end

function RepTracker:ToggleReset(info, value)
	self.db.faction.reset = value
end

function RepTracker:ScanFactions()
	local factions = {}
	local name, isHeader, isCollapsed
	factions["none"] = 0
	for i = 0, GetNumFactions() do
		_, _, _, _, _, _, _, _, isHeader, isCollapsed = GetFactionInfo(i)
		if isHeader and isCollapsed then
			ExpandFactionHeader(i)
		end
	end
	for i = 0, GetNumFactions() do
		name, _, _, _, _, _, _, _, isHeader = GetFactionInfo(i)
		if not isHeader and name ~= nil then
			factions[string.lower(name)] = i
		end
	end
	self.db.faction.factions = factions
end

function RepTracker:ZONE_CHANGED_NEW_AREA()
	self:ScanFactions()
	local faction = self:GetZone()
	if faction == nil then
		if self.db.faction.showInChat then
			self:Print(GetZoneText()..": no faction tracked")
			self:Print("To set the tracked faction for this zone, type: /rt track <faction>")
		end
		if self.db.faction.reset then
			SetWatchedFactionIndex(0)
		end
	else
		faction = string.lower(faction)
		if(self.db.faction.factions[faction] ~= nil) then
			SetWatchedFactionIndex(self.db.faction.factions[faction])
			if self.db.faction.showInChat then
				self:Print(GetZoneText()..": tracking "..faction.." reputation")
			end
		else
			self:Print("You have not made contact with that faction yet; tracking none")
			if self.db.faction.reset then
				SetWatchedFactionIndex(0)
			end
		end
	end
end

function RepTracker:GetZone()
	return self.db.faction.ZoneToFaction[GetZoneText()]
end

function RepTracker:SetZone(info, value)
	self:ScanFactions()
	if(self.db.faction.factions[string.lower(value)] ~= nil) then
		self.db.faction.ZoneToFaction[GetZoneText()] = string.lower(value)
		SetWatchedFactionIndex(self.db.faction.factions[string.lower(value)])
		if self.db.faction.showInChat then
			self:Print("Now tracking "..value.." reputation")
		end
	else
		self:Print("You have not made contact with that faction")
	end
end

function RepTracker:COMBAT_TEXT_UPDATE(arg1, messageType, faction, amount)
    if messageType ~= "FACTION" then
		-- we don't care about whatever this message is
		return
	end
	if (amount < 0) and (self.db.faction.dynamic) and (self.db.faction.decrease) then
		-- you lost <amount> reputation with <faction>
		self:changeBar(faction)
	elseif (self.db.faction.dynamic) then
		-- you gained <amount> reputation with <faction>
		self:changeBar(faction)
	end
end

function RepTracker:changeBar(faction)
	local oldName,_,_,_,_ = GetWatchedFactionInfo();
	for i=1, GetNumFactions() do
		local name,_,_,_,_,_,_,_,_,_ = GetFactionInfo(i);
		if (name == faction) then
			if not (name == oldName) and not (name == "alliance") and not (name == "horde") then
				SetWatchedFactionIndex(i)
				if (self.db.faction.sticky == false) then
					self:Print("Now tracking "..name.." reputation")
				end
				self.db.faction.lastWatched = i
				if self.db.faction.sticky then
					self:SetZone(oldName, faction)
				end
			end
		end
	end
end